#!/tools/gdaac/COTS/perl-5.8.5/bin/perl

=head1 NAME

synPGE.pl - Synthetic PGE 

=head1 SYNOPSIS

synPGE.pl 
[B<-p> I<PCF_file>] 
[B<-c> I<config_file>] 
[B<-h>]
   
=head1 ARGUMENTS 

=over 4

=item B<-p> I<PCF_file>

PCF must be specified via B<-p> option or PGS_PC_INFO_FILE environment variable.

=item B<-c> I<config_file> 

If config_file is not provided, the default ../synPGE.cfg will be used.

=item B<-m> I<metadata_template>

File with template for creating output metadata files.

=item B<-h>

Print usage and exit.

=back

=head1 DESCRIPTION

This program simulates the functionality of PGEs from the blackbox
standpoint.  Based on the information given in the PCF, tt verfies
the existance of standard and support input files.  It also 
generates standard and support output data files.

=head1 ALGORITHM

The sequence of steps is as follows:

1. Read in configuration file

2. Read in PCF file

3. For each standard/support/intermediate input files
   Check to make sure that it exists and readable

4. For each standard/support/intermediate output files
   Check to make sure that associated path exists and 
   writable. Write something to the file

5. Sleep for a configurable amount of time

=head1 AUTHOR

Michael Ta - NASA/GSFC-Raytheon ITSS, Code 902

=cut

# -@@@ synPGE.pl, Version 4.4.1
# SccsId: 2000/11/17 1.13
#####################################################################
# MODIFICATION
#
# 08/15/2000     First version
# 08/18/2000     MTa.  Added use strict and use Safe
# 09/11/2000     MTa,  Put in code to handle the possibility that 
#                the return from $pcf->input/output_product_files is 
#                not a hash with files, but potentially a hash with 
#                whitespace-separated LISTS of files as values
#
#####################################################################

# Declare packages
use strict;
use Getopt::Std;
use Cwd;
use File::Copy;
use File::Basename;
use S4P;
use S4P::PCF;
use S4PM;
use S4P::TimeTools;
use Safe;
use vars qw($opt_p $opt_c $opt_h $opt_m);
use vars qw($cfg_sleep_time $cfg_file_access_err $cfg_dir_access_err);
use vars qw($cfg_pcf_syntax_err $cfg_pcf_verifier_location);

# Declare variables
my $result;         	# status of the operation
my $timestamp;		# Time stamp, in YYYY:MM:DD-HH24:MM:SS
my $logstatusfile;
my $cur_dir;
my ($cmode, $amode) = ("C","A");

# Parse command arguments
my @cmd_args = @ARGV;
getopts('c:hm:p:');
if ($opt_h || (!$opt_p && !$ENV{'PGS_PC_INFO_FILE'})) {
   print STDERR "\nUsage: synPGE.pl [-p PCF_file] ";
   print STDERR "[-c config_file] [-h]\n\n";
   print STDERR "\tNOTE\n";
   print STDERR "\tPCF must be specified via PGS_PC_INFO_FILE environment variable or -p option\n\n";
   exit 100;
}

my $config_file = $opt_c || '../synPGE.cfg';
my $metadata_template = $opt_m || '../template.met';

# Set defaults
$cfg_sleep_time = 25;
$cfg_file_access_err  = 101;
$cfg_dir_access_err  = 102;
$cfg_pcf_syntax_err  = 103;
$cfg_pcf_verifier_location = '../pccheck_dir';

# Try to use configuration file if on the command line or default file exists
configure($config_file) if ($opt_c || -f $config_file);

# Check for file existence and readability and read in PCF file
my $pcf_file = $opt_p ? $opt_p: $ENV{'PGS_PC_INFO_FILE'};
my $pcf;
if ( -r $pcf_file && -s $pcf_file ) {
   # verify the correctness of the PCF  
#  $result = verify_PCF($pcf_file);
   # PCF file contains no error; read and parse PCF file
   $pcf = S4P::PCF::read_pcf($pcf_file);
   $pcf->read_file;
   $pcf->parse;
} else {
   S4P::perish($cfg_file_access_err,"File $pcf_file is non-existent or unreadable");
}

# Extract log files from the PCF and write to those files
my $r_logfiles = $pcf->log_files;
my %logfiles = %{$r_logfiles};
my $f;
foreach $f(sort keys %logfiles) {
    # print "\t$f:\t$logfiles{$f}\n";
    my $logfile = substr($logfiles{$f},2,length($logfiles{$f}));
    write_file($logfiles{$f}, $cmode, "Open $logfile file");
    if ($f eq "status") {
       $logstatusfile = $logfiles{$f};
    }
}

# Verity each standard input files
# print "\tStandard Input Data Files:\n";
verify_input_files($pcf->input_data_files);

# Verify each support input files
# print "\tSupport Input Data Files:\n";
verify_input_files($pcf->support_input_files);

# Verify each support input files
# print "\tIntermediate Input Data Files:\n";
verify_input_files($pcf->intermediate_input_files);

# Check for write permission of each directory associated 
# with standard output files. Also, write something to the file
verify_output_files($pcf->output_data_files);
verify_met_files($pcf->output_met_files, $pcf_file, $metadata_template);

# Check for write permission of each directory associated 
# with support output files. Also, write something to the file
verify_output_files($pcf->support_output_files);

# Check for write permission for LUNs 10252 and 10254, if they exist
my $getattr_temp = $pcf->product_input_files->{'10252.1'};
verify_output_file($getattr_temp) if $getattr_temp;
my $mcf_temp = $pcf->product_input_files->{'10254.1'};
verify_output_file($mcf_temp) if $mcf_temp;

# Check for write permission of each directory associated 
# with intermediate output files. Also, write something to the file
verify_output_files($pcf->intermediate_output_files);

# Sleep for a configurable amount of time
sleep $cfg_sleep_time;

exit 0;

# End of synPGE.pl

##################################################################################
# This routine write the message to the specified file
# Inputs:
#   File_name
#   Mode = Create (C) or Append (A); default = Append
#   Message
##################################################################################
sub write_file {
    my ($filename, $mode, $msg) = @_;
    if ($mode eq "C") {
        open (FH, "> $filename") || die("Failed to open $filename");
        chmod (0664, $filename);
    } else {
        open (FH, ">> $filename") || die("Failed to open $filename");
    }
    print FH "$msg\n";
    close FH;
}   # End of write_file

##################################################################################
# This routine verifies the syntax of the PCF file using pccheck.sh 
# Inputs:
#   PCF file name
##################################################################################
sub verify_PCF {
   my ($filename) = @_;
    
   my $cur_dir = getcwd();		# get current directory
   my $ret = copy($filename,"/var/tmp/pcf_$$.file");
   chdir $cfg_pcf_verifier_location;
   my @results = `pccheck.sh -i /var/tmp/pcf_$$.file`;
   if ($?) {
       S4P::logger("WARN", "Error returned from pccheck.sh: $?");
   }
   unlink "/var/tmp/pcf_$$.file";
   chdir $cur_dir;
   # Check the results; search for #errors/warnings
   chop @results;
   for (my $i = 0; $i < @results; $i++) {
       if ($results[$i] =~ /Errors found:/) {
          my ($parm, $error) = split ("\s*:\s*", $results[$i]);
          if ($error > 0) {
             S4P::perish($cfg_pcf_syntax_err, "The $pcf_file contains error\n");
          }
       } 
       if  ($results[$i] =~ /Warnings found:/) {
          my ($parm, $warn) = split ("\s*:\s*", $results[$i]);
          if ($warn > 0) {
             S4P::logger('WARNING', "Verification of $pcf_file yields $warn warnings\n");
          }
       } 
       elsif ($results[$i] !~ /^\s*$/) {
             S4P::logger('INFO', "pccheck.sh $results[$i]\n");
       }
   }   # End of for loop
   return 0;
}  # End of verify_PCF

##################################################################################
# This routine checks for existence, file size, and readability of each given file
##################################################################################
sub verify_input_files {
   my ($filelist) = @_;

   my ($lun, $file, @files);
   foreach $lun(keys %{$filelist}) {
       # Screen out LUNs 10252 and 10254
       next if ($lun =~ /^1025[24]\./);

       # Can be more than one file in the field, split by whitespace
       @files = split(' ', $filelist->{$lun});
       foreach $file(@files) {
           # Interpolate tilde ~ to $PGSHOME env variable
           $file =~ s/^~/$ENV{'PGSHOME'}/ if $ENV{'PGSHOME'};
           if ( !(-r $file) || !(-s $file) ) {
               write_file($logstatusfile, $amode, 
                   "$file is non-existent|unreadable|zero-size");
               S4P::perish($cfg_file_access_err, 
                   "$file is non-existent|unreadable|zero-size");
           }
       }
   }
}   # End of verify_input_files

##################################################################################
# This routine checks for write permission of each directory associated 
# with given files. The routine also writes something to the file 
##################################################################################
sub verify_output_files {
   my @files = map {split ' '} values %{shift @_};
   map {verify_output_file($_)} @files;
}  # End of verify_output_files
sub verify_output_file {
    my $file = shift;
    my $dirname = dirname($file);
    my $filename = basename($file);
    if ( (-d $dirname) && (-w $dirname) ) {
        write_file($file, $cmode, "Hello, world, I am your data file.");
    } 
    else {
        write_file($logstatusfile, $amode, "$file is unwritable");
        S4P::perish($cfg_dir_access_err, "$file is unwriteable");
    }
}

##################################################################################
# This routine reads in configuration parameters
# The routine is based on run_pge's configure routine
##################################################################################
sub configure {
   my $file = shift;
   if ($file) {
      # Read in configuration file
      my $compartment = new Safe "CFG";
      $compartment->share('$cfg_sleep_time');
      $compartment->share('$cfg_file_access_err');
      $compartment->share('$cfg_dir_access_err');
      $compartment->share('$cfg_pcf_syntax_err');
      $compartment->share('$cfg_pcf_verifier_location');
      $compartment->rdo($file) or
          S4P::perish(11, "Cannot read config file $file in safe mode: $!\n");
      $main::cfg_sleep_time = $CFG::cfg_sleep_time if $CFG::cfg_sleep_time;
      $main::cfg_file_access_err = $CFG::cfg_file_access_err if $CFG::cfg_file_access_err;
      $main::cfg_dir_access_err = $CFG::cfg_dir_access_err if $CFG::cfg_dir_access_err;
      $main::cfg_pcf_syntax_err = $CFG::cfg_pcf_syntax_err if $CFG::cfg_pcf_syntax_err;
      $main::cfg_pcf_verifier_location = $CFG::cfg_pcf_verifier_location if $CFG::cfg_pcf_verifier_location;
    }
}


##################################################################################
# This routine checks for write permission of each directory associated
# with given files. The routine also writes something to the file
##################################################################################
sub verify_met_files {
   my ($filelist, $pcf_file, $metadata_template) = @_;


   my @files = map {split ' '} values %{$filelist};
   my ($dirname, $filename);
   for ( my $i = 0; $i < @files; $i++) {
       # print "\t\t$i: $files[$i]\n";
       $dirname = dirname($files[$i]);
       $filename = basename($files[$i]);

       my ($esdt, $platform, $begin, $version, $prod_time, $suffix) = S4PM::parse_modis_filename($filename);
       my $begin_yy = substr($begin,0,4);
       my $begin_jdate = substr($begin,4,3);
       my ($begin_month, $begin_day) = jdate2day($begin_jdate,$begin_yy);
       my $begin_hh = substr($begin, 7,2);
       my $begin_mm = substr($begin, 9,2);
       my $begin_ss;
       if (length($begin) > 11) {
          $begin_ss = substr($begin,11,2);
       } else {
          $begin_ss = "00";
       }

       my $start = "$begin_yy-$begin_month-$begin_day" . "T". "$begin_hh:$begin_mm:$begin_ss" ."Z";
       $esdt = "\"" . $esdt . "\"";
       my $start_date = "\"" . substr($start,0,10) . "\"";
       my $start_time = "\"" . substr($start,11,8) . "\"";
       my $end = S4P::TimeTools::CCSDSa_DateAdd($start, 300);
       my $end_date = "\"" . substr($end,0,10) . "\"";
       my $end_time = "\"" . substr($end,11,8) . "\"";

       if ( (-d $dirname) && (-w $dirname) ) {
          open (FH, "> $files[$i]") or die("Failed to open $filename");
          chmod (0664, $files[$i]);
          open(FH_IN, $metadata_template) or 
             die("failed to open metadata template $metadata_template");
          my @all_lines = <FH_IN>;
          foreach my $line (@all_lines) {
             if ( $line =~ /INSERT_ESDT_HERE/ ) {
                $line =~ s/\"INSERT_ESDT_HERE\"/$esdt/;
             } elsif ( $line =~ /INSERT_END_TIME_HERE/ ) {
                $line =~ s/\"INSERT_END_TIME_HERE\"/$end_time/;
             } elsif ( $line =~ /INSERT_END_DATE_HERE/ ) {
               $line =~ s/\"INSERT_END_DATE_HERE\"/$end_date/;
             } elsif ( $line =~ /INSERT_BEGIN_TIME_HERE/ ) {
               $line =~ s/\"INSERT_BEGIN_TIME_HERE\"/$start_time/;
             } elsif ( $line =~ /INSERT_BEGIN_DATE_HERE/ ) {
               $line =~ s/\"INSERT_BEGIN_DATE_HERE\"/$start_date/;
             }
             print FH $line;
          }
          close(FH_IN);
          close(FH);
       } else {
         write_file($logstatusfile, $amode, "$files[$i] is not-directory|unwritable");
         S4P::perish($cfg_dir_access_err, "$files[$i] is not-directory|unwriteable");
       }
   }
}  # End of verify_met_files

#########################################################################

sub jdate2day {
    my ($julianday,$year) = @_;

    my @dayspermon = (31,28,31,30,31,30,31,31,30,31,30,31);
    my @mon = ("01","02","03","04","05","06","07","08","09","10","11","12");
    if ( (($year % 4 == 0) && ($year % 100 != 0)) || ($year % 400 ==0 ) ){
        $dayspermon[1] = 29;
        }
    my $i = 0;
    my $date = $julianday;

    for(; $date > $dayspermon[$i]; $date -= $dayspermon[$i++]){ }

    if (length($date) == 1) { $date = "0$date"; }
    if (length($date) < 2) {$date = "0$date";}
    if (length($mon[$i]) < 2) {$mon[$i] = "0$mon[$i]";}
    ($mon[$i], $date);
} # end of jdate2day

##############################################################################
