#!/usr/bin/perl

=head1 NAME

s4pm_run_test.pl - create a simple string and run it

=head1 SYNOPSIS

s4pm_run_test.pl
[-b]
[-r]

=head1 DESCRIPTION

This interactive script can be used to check out a new installation of S4PM. 
The script creates a simple S4PM string configured for two simulation 
algorithms. The output of the first algorithm is used as input to the second. 
Thus, the algorithms are chained. 

This script assumes that S4PM has been correctly installed.

The only inputs needed by the script are the locations of the S4PM binaries
and configuration files. After these are entered, the rest of the script
runs automatically. At each step along the process, the user is prompted
to continue or quit.

NOTE: The sample test string is created under the current working directory. 
Thus, do NOT run this script from within a directory containing real live 
Stringmaker configuration files.

The following steps are carried out by this script:

=over 4

=item 1

Based upon information entered and obtained from the user's environment,
the script will use templates to generate the necessary Stringmaker 
configuration files.

=item 2

A sample Distribution Notification is generated based upon a template.

=item 3

Stringmaker is run to create the string under the current working directory.

=item 4

Sample data is copied into the INPUT disk pool of that new string and a 
Distribution Notification is placed as a work order into the Receive DN
station.

=item 5

The stations in the string are turned on and the processing begins.

=back

=head1 ARGUMENTS

=over 4

=item -b

Run the script without pausing for user interaction. By default, all the steps 
in the process are run after prompting the user. With the -b option, the user
is not prompted at each step along the way. Instead, the test blasts right
through.

=item -r

The -r option is used to merely rerun the test without rebuilding the string
from scratch. It will reset the current string to its initial state and then
run it.

=back

=head1 AUTHOR

Stephen Berrick, NASA/GSFC, Code 610.2

=cut

################################################################################
# $Id: s4pm_run_test.pl,v 1.2 2006/09/12 20:31:39 sberrick Exp $
# -@@@ S4PM, Version $Name:  $
################################################################################

use Cwd;
use File::Basename;
use File::Copy;
use Getopt::Std;
use vars qw($opt_r $opt_b);

my $pwd = cwd();
my $ans;
my $bindir;
my $cfgdir;

getopts('rb');

unless ( $opt_r ) {

### Intro

    print "\n";
    print "This script will build a test S4PM string configured with 2 simple algorithms.\n";
    print "The entire process is automatic and should only take a few minutes.\n";

### Ask for some information

    $bindir = guess_dir();
    unless ( $bindir ) {
        die "You need to set your PATH environment variable to include the location where the\nS4PM binaries have been installed.\n\n";
    } else {
        $cfgdir = $bindir;
    }

### Obtain needed information

    my $user = getlogin();
    my $host = `hostname`;
    chomp($host);
    if ( $host =~ /^([^.]+)\..*$/ ) {
        $host = $1;
    }

    my $s4pm_root = "$pwd/s4pm";
    my $algorithm_root = "$pwd/algorithms";

    is_continue("About to generate Stringmaker configuration files.");

### Generate a host configuration file

    $/ = undef;
    my $cfg = "s4pm_stringmaker_host.template";
    open(FILE, $cfg) or die "Failed to open $cfg for read: $!\n\n";
    my $slurp = <FILE>;
    close FILE;
    $slurp =~ s/INSERT_BINDIR_HERE/$bindir/;
    $slurp =~ s/INSERT_CFGDIR_HERE/$cfgdir/;
    $slurp =~ s/INSERT_HOST_HERE/$host/;
    $slurp =~ s/INSERT_S4PM_ROOT_HERE/$s4pm_root/;
    my $hostcfg = "$host.cfg";
    open(OUT, ">$hostcfg") or 
        die "Failed to open $hostcfg for write: $!\n\n";
    print OUT $slurp;
    close OUT;

### Generate a global configuration file

    my $cfg = "s4pm_stringmaker_global.template";
    open(FILE, $cfg) or die "Failed to open $cfg for read: $!\n\n";
    my $slurp = <FILE>;
    close FILE;
    $slurp =~ s/INSERT_USER_HERE/$user/;
    my $globalcfg = "s4pm_stringmaker_global.cfg";
    open(OUT, ">$globalcfg") or 
        die "Failed to open $globalcfg for write: $!\n\n";
    print OUT $slurp;
    close OUT;

### Generate a data types configuration file

    my $cfg = "s4pm_stringmaker_datatypes.template";
    open(FILE, $cfg) or die "Failed to open $cfg for read: $!\n\n";
    my $slurp = <FILE>;
    close FILE;
    $slurp =~ s/INSERT_BINDIR_HERE/$bindir/g;
    my $datatypescfg = "s4pm_stringmaker_datatypes.cfg";
    open(OUT, ">$datatypescfg") or 
        die "Failed to open $datatypescfg for write: $!\n\n";
    print OUT $slurp;
    close OUT;

### Generate a string configuration file

    my $cfg = "s4pm_stringmaker_string.template";
    open(FILE, $cfg) or die "Failed to open $cfg for read: $!\n\n";
    my $slurp = <FILE>;
    close FILE;
    $slurp =~ s/INSERT_ALGORITHM_ROOT_HERE/$algorithm_root/;
    my $stringcfg = "FIRST_TEST_STRING.cfg";
    open(OUT, ">$stringcfg") or 
        die "Failed to open $stringcfg for write: $!\n\n";
    print OUT $slurp;
    close OUT;
    $/ = "\n";

### Generate a DN

    is_continue("About to generate a fake Distribution Notification.");

    $/ = undef;
    my $cfg = "S4PM.DN.1324.template";
    open(FILE, $cfg) or die "Failed to open $cfg for read: $!\n\n";
    my $slurp = <FILE>;
    close FILE;
    my $dir = "$pwd/s4pm/test1/stations/DATA/INPUT";
    $slurp =~ s/INSERT_FTPDIR_HERE/$dir/;
    my $dn = "S4PM.DN.1324";
    open(OUT, ">$dn") or 
        die "Failed to open $dn for write: $!\n\n";
    print OUT $slurp;
    close OUT;
    $/ = "\n";

### Run Stringmaker

    is_continue("About to run Stringmaker. NOTE: You will see a lot of messages scroll\nby.");

    my $cmd = "$bindir/s4pm_stringmaker.pl -c -s FIRST_TEST_STRING.cfg";
    my ($errstr, $rc) = exec_system($cmd);
    if ($rc) {
        die "Failed to Stringmaker: $errstr\n\n";
    }

} else {	# -r option is invoked

### Intro

    print "\n";
    print "This script will reset the current S4PM string to its initial state and then\n";
    print "rerun it.\n";

    is_continue("About to reset the string.");

}

# Copy data into DATA/INPUT 

is_continue("About to prepare test data for algorithms.");

copy("L1BRAD.A2005292.1720.005.2005293111603.hdf",
     "s4pm/test1/stations/DATA/INPUT");
copy("L1BRAD.A2005292.1720.005.2005293111603.hdf.met",
     "s4pm/test1/stations/DATA/INPUT");

# Copy DN into Receive DN station

copy("S4PM.DN.1324", "s4pm/test1/stations/receive_dn");

# Bring up S4PM Monitor

is_continue("About to bring up the S4PM Monitor. This may take a few seconds.");

chdir("s4pm/test1/stations/");
my ($errstr, $rc) = exec_system("./s4pm_start.ksh");
if ($rc) {
    die "Failed to start up the S4PM Monitor: $errstr\n\n";
}
sleep(8);

# Start the string up

is_continue("All is prepared. About to turn on the stations.");

chdir("s4pm/test1/stations/");
my ($errstr, $rc) = exec_system("$bindir/s4pstart.ksh");
if ($rc) {
    die "Failed to turn on string: $errstr\n\n";
}

# But turn off the Export station

chdir("s4pm/test1/stations/");
my ($errstr, $rc) = exec_system("touch export/DO.STOP.wo");
if ($rc) {
    die "Failed to turn on string: $errstr\n\n";
}

# Now, wait until 2 EXPORT work orders show up in Export station, but
# don't wait forever

my $maxtimer = 10000;
my $t = 0;
my $both_found = 0;
do {
    my @files = glob("export/DO.EXPORT*.wo");
    my $c = scalar(@files);
    if ( scalar(@files) == 2 ) {
        $both_found = 1;
    } 
    sleep(5);
    $t++;
} while ( ! $both_found and $t < $maxtimer );
    
if ( ! $both_found ) {
    print "timer expired. Something is wrong!\n";
    die "\nTimer expired waiting for the two expected EXPORT work orders. Something must\nbe wrong. This test has FAILED.\n\n";
}

# Do a checksum and size check of the output files

my @hdf_file = glob("DATA/L1BLST/L1BLST*.hdf");
my $cs = `cksum $hdf_file[0]`;
my @parts = split(/\s/, $cs);
$cs = $parts[0] . " " . $parts[1];
if ( $cs ne "1039613180 35" ) {
    print "Computed checksum: [$cs] not equal to expected: [1039613180 35]\n";
    die "\nChecksum mismatch for the L1BLST HDF file. This test has FAILED.\n\n";
}

my @met_file = glob("DATA/L1BLST/L1BLST*.met");
my $cs = `cksum $met_file[0]`;
my @parts = split(/\s/, $cs);
$cs = $parts[0] . " " . $parts[1];
if ( $cs ne "3314109718 17974" ) {
    print "Computed checksum: [$cs] not equal to expected: [3314109718 17974]\n";
    die "\nChecksum mismatch for the L1BLST met file. This test has FAILED.\n\n";
}

my @hdf_file = glob("DATA/L1BBRDF/L1BBRDF*.hdf");
my $cs = `cksum $hdf_file[0]`;
my @parts = split(/\s/, $cs);
$cs = $parts[0] . " " . $parts[1];
if ( $cs ne "1039613180 35" ) {
    print "Computed checksum: [$cs] not equal to expected: [1039613180 35]\n";
    die "\nChecksum mismatch for the L1BBRDF HDF file. This test has FAILED.\n\n";
}

my @met_file = glob("DATA/L1BBRDF/L1BBRDF*.met");
my $cs = `cksum $met_file[0]`;
my @parts = split(/\s/, $cs);
$cs = $parts[0] . " " . $parts[1];
if ( $cs ne "1820616528 17242" ) {
    print "Computed checksum: [$cs] not equal to expected: [1820616528 17242]\n";
    die "\nChecksum mismatch for the L1BBRDF met file. This test has FAILED.\n\n";
}

print "################################################################################\n";
print "\nCongratulations! The data produced looks good!\n\n";
print "################################################################################\n";
sleep(3);

# Turn on Export station

is_continue("About to turn on the Export station to complete processing.");
sleep(3);

my ($errstr, $rc) = exec_system("$bindir/s4pstart.ksh");
if ($rc) {
    die "Failed to turn on Export station: $errstr\n\n";
}

# Verify final state of string

sleep(10);

my $maxtimer = 10000;
my $t = 0;
my $is_empty = 0;
do {
    my @files = glob("DATA/L1BLST/*");
    my $c = scalar(@files);
    if ( scalar(@files) == 1 ) {
        $is_empty = 1;
    } 
    sleep(5);
    $t++;
} while ( ! $is_empty and $t < $maxtimer );

if ( ! $is_empty ) {
    die "\nTimer expired waiting for DATA/L1BLST directory to empty out.\n\n";
}

my $maxtimer = 10000;
my $t = 0;
my $is_empty = 0;
do {
    my @files = glob("DATA/L1BBRDF/*");
    my $c = scalar(@files);
    if ( scalar(@files) == 1 ) {
        $is_empty = 1;
    } 
    sleep(5);
    $t++;
} while ( ! $is_empty and $t < $maxtimer );

if ( ! $is_empty ) {
    die "\nTimer expired waiting for DATA/L1BBRDF directory to empty out.\n\n";
}

my $maxtimer = 10000;
my $t = 0;
my $is_empty = 0;
do {
    my @files = glob("DATA/INPUT/*");
    my $c = scalar(@files);
    if ( scalar(@files) == 0 ) {
        $is_empty = 1;
    } 
    sleep(5);
    $t++;
} while ( ! $is_empty and $t < $maxtimer );

if ( ! $is_empty ) {
    die "\nTimer expired waiting for DATA/INPUT directory to empty out.\n\n";
}

print "################################################################################\n";
print "# Data has been successfully exported and cleaned up.\n";
print "################################################################################\n";

# Shut down and say good-bye

    my $cmd = "$bindir/s4pshutdown.pl -r";
    my ($errstr, $rc) = exec_system($cmd);
    if ($rc) {
        die "Failed to shutdown string: $errstr\n\n";
    }

print "\n\n";
print "################################################################################\n";
print "################################################################################\n";
print "# CONGRATULATIONS! Every thing is just Ok!\n";
print "################################################################################\n";
print "################################################################################\n";

sub exec_system {

    # "localize" SIG{CHLD} to get around Sun problem of conflicting
    # signal handlers
    my $save_sig = $SIG{CHLD};
    undef($SIG{CHLD});
    my $rc = 0xffff & system( @_ );
    $SIG{CHLD} = $save_sig;
    # Success
    if ($rc == 0) {
        return wantarray ? ('',0) : 0;
    }
    # Command failed to execute
    elsif ($rc == 0xff00) {
        return wantarray ? ("Command $_[0] failed: $rc", $rc) : $rc;
    }
    # Command executed, but exited with a non-zero status
    elsif ($rc > 0x80) {
        $rc >>= 8;
        return wantarray ? ("Job failed with exit $rc", $rc) : $rc;
    }
    # Command terminated abnormally (coredump or signal)
    else {
        if ($rc & 0x80) {
            $rc &= ~0x80;
            return wantarray ?
                ("Command $_[0] died with coredump from signal $rc", $rc) : $rc;
        }
        else {
            return wantarray ? ("Command $_[0] died with signal $rc", $rc) : $rc
;
        }
    }
}

sub is_continue {

    my $prompt = shift;

    unless ( $opt_b ) {
        print "\n$prompt Ok? (y/n) [y] ";
        $ans = <STDIN>;
        chomp($ans);
        if ( $ans =~ /^N/ or $ans =~ /^n/ ) { 
            print "\nOk. Good-bye.\n\n";
            exit 0;
        }
    } else {
        print "################################################################################\n";
        print "# $prompt\n";
        print "################################################################################\n";
    }

}

sub guess_dir {

    my $path = $ENV{'PATH'};
    my @paths = split(/:/, $path);
    foreach my $p ( @paths ) {
        if ( -e "$p/s4pm_run_algorithm.pl" and
             -e "$p/s4pm_find_data.pl" and 
             -e "$p/s4pm_stringmaker_derived.cfg" ) {
            return $p;
        }
    }

    return;

}

